@@ -73,6 +73,7 @@ lasso_release_nid_and_session_index(struct _NidAndSessionIndex *nid_and_session_
 	lasso_release_gobject(nid_and_session_index->name_id);
 	lasso_release_string(nid_and_session_index->session_index);
 	lasso_release_string(nid_and_session_index->assertion_id);
+	lasso_release(nid_and_session_index);
 }
 
 void
@@ -857,7 +858,7 @@ init_from_xml_nid_and_session_index(LassoNode *node, xmlNode *nid_and_session_in
 	xmlChar *provider_id = NULL;
 	xmlChar *assertion_id = NULL;
 	xmlNode *nid;
-	LassoNode *name_id;
+	LassoNode *name_id = NULL;
 	struct _NidAndSessionIndex *nid_and_session_index;
 
 	provider_id = xmlGetProp(nid_and_session_index_node, BAD_CAST PROVIDER_ID);
@@ -881,6 +882,7 @@ cleanup:
 	lasso_release_xml_string(session_index);
 	lasso_release_xml_string(provider_id);
 	lasso_release_xml_string(assertion_id);
+	lasso_release_gobject(name_id);
 }
 
 static int
